@@ -10,16 +10,16 @@
  * to get information about the current tool location and relocate
  * the box.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -51,12 +51,18 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  * root cell def.  If the box is placed in one window, it will
  * be displayed in all compatible windows.  The box may not have
  * one corner placed in one window and other corners in
- * incompatible windows.  
+ * incompatible windows.
  */
 
 static CellDef *boxRootDef = NULL;	/* CellDef for the box */
 static Rect boxRootArea;		/* Root def coords */
-static Point crosshairPos;		/* Crosshair position */
+
+typedef struct _crosshairRec {
+    Point    pos;	/* Crosshair position */
+    CellDef *def;	/* CellDef that crosshair position references */
+} CrosshairRec;
+
+static CrosshairRec curCrosshair;		/* Crosshair position */
 
 /*
  * If the following is DBW_SNAP_USER, the box gets snapped to the user's
@@ -69,7 +75,7 @@ int DBWSnapToGrid = DBW_SNAP_LAMBDA;
 
 extern int DBWToolDraw();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -117,10 +123,10 @@ toolFindPoint(p, rootPoint, rootArea)
     if (DBWSnapToGrid != DBW_SNAP_INTERNAL)
 	ToolSnapToGrid(WindCurrentWindow, rootPoint, rootArea);
     return WindCurrentWindow;
-    
+
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *	ToolGetPoint --
@@ -153,13 +159,13 @@ ToolGetPoint(rootPoint, rootArea)
 {
     extern TxCommand *WindCurrentCmd;
 
-    if (WindCurrentCmd == NULL) 
+    if (WindCurrentCmd == NULL)
 	return NULL;
     else
 	return toolFindPoint(&WindCurrentCmd->tx_p, rootPoint, rootArea);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * ToolGetBox --
@@ -203,7 +209,7 @@ ToolScaleBox(scalen, scaled)
 
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * ToolGetBoxWindow --
@@ -251,7 +257,7 @@ ToolGetBoxWindow(rootArea, pMask)
     toolMask = 0;
     window = NULL;
     if (boxRootDef != NULL)
-	(void) WindSearch(DBWclientID, (ClientData) NULL, (Rect *) NULL, 
+	(void) WindSearch(DBWclientID, (ClientData) NULL, (Rect *) NULL,
 	    toolWindowSave, (ClientData) &window);
     if ((window != NULL) && (rootArea != NULL)) *rootArea = boxRootArea;
     if (pMask != NULL) *pMask = toolMask;
@@ -272,7 +278,7 @@ toolWindowSave(window, clientData)
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -298,12 +304,12 @@ bool
 ToolGetEditBox(rect)
     Rect *rect;
 {
-    if (boxRootDef == NULL) 
+    if (boxRootDef == NULL)
     {
 	TxError("Box must be present\n");
 	return FALSE;
     }
-    if (EditRootDef != boxRootDef) 
+    if (EditRootDef != boxRootDef)
     {
 	TxError("The box isn't in a window on the edit cell.\n");
 	return FALSE;
@@ -313,7 +319,7 @@ ToolGetEditBox(rect)
     return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *	ToolGetCorner --
@@ -383,15 +389,16 @@ ToolGetCorner(screenPoint)
  *
  * Side effects:
  *	Initializes static memory.
- *	
+ *
  * ----------------------------------------------------------------------------
  */
 
 void
 dbwCrosshairInit()
 {
-    crosshairPos.p_x = MINFINITY;
-    crosshairPos.p_y = MINFINITY;
+    curCrosshair.pos.p_x = MINFINITY;
+    curCrosshair.pos.p_y = MINFINITY;
+    curCrosshair.def = (CellDef *)NULL;
     DBWHLAddClient(DBWDrawCrosshair);
 }
 
@@ -411,8 +418,8 @@ dbwCrosshairInit()
  */
 
 void
-dbwRecordCrosshairYPos(window, erase)
-    MagWindow *window;
+dbwRecordCrosshairYPos(def, erase)
+    CellDef *def;
     bool erase;			/* TRUE means crossair is being erased from its
 				 * current position.  FALSE means the crosshair
 				 * is being added at a new position.
@@ -422,13 +429,13 @@ dbwRecordCrosshairYPos(window, erase)
 
     xwire.r_xbot = MINFINITY;
     xwire.r_xtop = INFINITY;
-    xwire.r_ybot = xwire.r_ytop = crosshairPos.p_y;
-    DBWHLRedraw(WINDOW_DEF(window), &xwire, erase);
+    xwire.r_ybot = xwire.r_ytop = curCrosshair.pos.p_y;
+    DBWHLRedraw(def, &xwire, erase);
 }
 
 void
-dbwRecordCrosshairXPos(window, erase)
-    MagWindow *window;
+dbwRecordCrosshairXPos(def, erase)
+    CellDef *def;
     bool erase;			/* TRUE means crossair is being erased from its
 				 * current position.  FALSE means the crosshair
 				 * is being added at a new position.
@@ -438,8 +445,8 @@ dbwRecordCrosshairXPos(window, erase)
 
     xwire.r_ybot = MINFINITY;
     xwire.r_ytop = INFINITY;
-    xwire.r_xbot = xwire.r_xtop = crosshairPos.p_x;
-    DBWHLRedraw(WINDOW_DEF(window), &xwire, erase);
+    xwire.r_xbot = xwire.r_xtop = curCrosshair.pos.p_x;
+    DBWHLRedraw(def, &xwire, erase);
 }
 
 /*
@@ -469,10 +476,17 @@ DBWDrawCrosshair(window, plane)
 {
     Point p;
 
-    /* Crosshair is disabled by setting to MINFINITY */
-    if (crosshairPos.p_x == MINFINITY) return;
+    /* Unlike most highlights, which are related to the database and	*/
+    /* therefore drawn on all windows, the crosshair only exists in	*/
+    /* the cell where the pointer is active, and in any other window	*/
+    /* with the same root cell.  It is therefore necessary to check	*/
+    /* if the root def of the window matches the root def of the window	*/
+    /* with active focus.  Otherwise, translating the crosshair		*/
+    /* position to window coordinates makes no sense.			*/
 
-    WindPointToScreen(window, &crosshairPos, &p);
+    if (WINDOW_DEF(window) != curCrosshair.def) return;
+
+    WindPointToScreen(window, &(curCrosshair.pos), &p);
 
     GrSetStuff(STYLE_YELLOW1);
     if (p.p_x > window->w_screenArea.r_xbot &&
@@ -493,7 +507,7 @@ DBWScaleCrosshair(scalen, scaled)
     int scalen;
     int scaled;
 {
-    DBScalePoint(&crosshairPos, scalen, scaled);
+    DBScalePoint(&(curCrosshair.pos), scalen, scaled);
 }
 
 /*
@@ -515,24 +529,43 @@ DBWSetCrosshair(window, pos)
     MagWindow *window;
     Point *pos;			/* New crosshair location in coords of rootDef. */
 {
-    if (crosshairPos.p_x != pos->p_x)
+    bool needUpdate = FALSE;
+
+    if (WINDOW_DEF(window) != curCrosshair.def) needUpdate = TRUE;
+
+    /* Erase */
+
+    if (needUpdate || (curCrosshair.pos.p_x != pos->p_x))
     {
 	/* Record the old and area of the vertical line for redisplay. */
-	dbwRecordCrosshairXPos(window, TRUE);
-
-	/* Update the crosshair location. */
-	crosshairPos.p_x = pos->p_x;
-
-	/* Record the new area for redisplay. */
-	dbwRecordCrosshairXPos(window, FALSE);
+	dbwRecordCrosshairXPos(curCrosshair.def, TRUE);
     }
 
     /* Do the same thing for the horizontal crosshair line */
-    if (crosshairPos.p_y != pos->p_y)
+    if (needUpdate || (curCrosshair.pos.p_y != pos->p_y))
     {
-	dbwRecordCrosshairYPos(window, TRUE);
-	crosshairPos.p_y = pos->p_y;
-	dbwRecordCrosshairYPos(window, FALSE);
+	dbwRecordCrosshairYPos(curCrosshair.def, TRUE);
+    }
+
+    /* Record the CellDef that this position is referenced to. */
+    if (needUpdate) curCrosshair.def = WINDOW_DEF(window);
+
+    /* Draw */
+
+    if (curCrosshair.pos.p_x != pos->p_x)
+    {
+	/* Update the crosshair location. */
+	curCrosshair.pos.p_x = pos->p_x;
+
+	/* Record the new area for redisplay. */
+	dbwRecordCrosshairXPos(curCrosshair.def, FALSE);
+    }
+
+    /* Do the same thing for the horizontal crosshair line */
+    if (curCrosshair.pos.p_y != pos->p_y)
+    {
+	curCrosshair.pos.p_y = pos->p_y;
+	dbwRecordCrosshairYPos(curCrosshair.def, FALSE);
     }
 }
 
@@ -584,7 +617,7 @@ dbwRecordBoxArea(erase)
 	DBWHLRedraw(boxRootDef, &side, erase);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -660,7 +693,7 @@ DBWDrawBox(window, plane)
      * blob, then don't do the widening.  This is to make the box more
      * useable when features are very small.
      */
-    
+
     if (((screenArea.r_xtop != screenArea.r_xbot) &&
 	    (screenArea.r_xtop < screenArea.r_xbot + 4))
 	    || ((screenArea.r_ytop != screenArea.r_ybot) &&
@@ -716,7 +749,7 @@ dbwBoxAlways1()
 {
     return 1;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *	DBWSetBox --
@@ -756,7 +789,32 @@ DBWSetBox(rootDef, rect)
 
     dbwRecordBoxArea(FALSE);
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ * DBWResetBox() ---
+ *
+ *	Make sure that boxRootDef is set to NULL if it is equal to the
+ *	specified CellDef.  This is used by the cell delete function to
+ *	make sure that if an edit cell is deleted, the boxRootDef is not
+ *	pointing to an invalid area of memory.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Global variable boxRootDef may be set to NULL.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBWResetBox(CellDef *def)
+{
+    if (def == boxRootDef)
+	boxRootDef = NULL;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *	ToolMoveBox --
@@ -766,7 +824,7 @@ DBWSetBox(rootDef, rect)
  *	the box corner is snapped to the user's grid (set with the :grid
  *	command) if DBWSnapToGrid is DBW_SNAP_USER.  If DBWSnapToGrid is
  *	DBW_SNAP_LAMBDA, the box corner is snapped to the nearest integer
- *	lambda value. 
+ *	lambda value.
  *
  * Results:
  *	None.
@@ -857,7 +915,7 @@ ToolMoveBox(corner, point, screenCoords, rootDef)
     DBWSetBox(newDef, &newArea);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *	ToolMoveCorner --
@@ -930,7 +988,7 @@ ToolMoveCorner(corner, point, screenCoords, rootDef)
      * current box root def, then just move the whole durned box.
      * Also move the whole box if a weird corner is specified.
      */
-    
+
     if ((newDef != oldDef) || (corner < 0) || (corner > TOOL_TL))
     {
 	ToolMoveBox(corner, &p, FALSE, newDef);
@@ -963,7 +1021,7 @@ ToolMoveCorner(corner, point, screenCoords, rootDef)
     /* If the movement turned the box inside out, turn it right
      * side out again.
      */
-    
+
     if (newArea.r_xbot > newArea.r_xtop)
     {
 	tmp = newArea.r_xtop;
@@ -979,7 +1037,7 @@ ToolMoveCorner(corner, point, screenCoords, rootDef)
 
     DBWSetBox(newDef, &newArea);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
