@@ -81,6 +81,7 @@ ESGenerateHierarchy(inName, flags)
     hc.hc_hierName = NULL;
     hc.hc_trans = GeoIdentityTransform;
     hc.hc_x = hc.hc_y = 0;
+
     EFHierSrDefs(&hc, esMakePorts, NULL);
     EFHierSrDefs(&hc, NULL, NULL);	/* Clear processed */
 
@@ -350,6 +351,13 @@ esOutputHierResistor(hc, dev, scale, term1, term2, has_model, l, w, dscale)
     /* term1=gate term2=source by the above code.	 */
     /* extracted units are Ohms; output is in Ohms 	 */
 
+    if ((term1->dterm_node == NULL) || (term2->dterm_node == NULL))
+    {
+	TxError("Error:  Resistor %s missing terminal node!\n",
+			EFDevTypes[dev->dev_type]);
+	return;
+    }
+
     spcdevOutNode(hc->hc_hierName, term1->dterm_node->efnode_name->efnn_hier,
 		"res_top", esSpiceF);
     spcdevOutNode(hc->hc_hierName, term2->dterm_node->efnode_name->efnn_hier,
@@ -405,6 +413,7 @@ subcktHierVisit(use, hierName, is_top)
     EFNode *snode;
     EFNodeName *nodeName;
     bool hasports = FALSE;
+    bool isStub;
 
     /* Avoid generating records for circuits that have no ports.	*/
     /* These are already absorbed into the parent.  All other		*/
@@ -431,6 +440,13 @@ subcktHierVisit(use, hierName, is_top)
 	    break;
 	}
 
+    /* Same considerations as at line 1831 for determining if the cell	*/
+    /* has been folded into the parent and should not be output.	*/
+
+    isStub = ((def->def_flags & DEF_ABSTRACT) && esDoBlackBox) ?  TRUE : FALSE;
+    if ((!is_top) && (def->def_flags & DEF_NODEVICES) && (!isStub))
+        return 0;
+
     if (hasports || is_top)
 	return subcktVisit(use, hierName, is_top);
     else if (def->def_flags & DEF_NODEVICES)
@@ -485,7 +501,7 @@ spcdevHierVisit(hc, dev, scale)
     EFNode  *subnode, *snode, *dnode, *subnodeFlat = NULL;
     int l, w, i, parmval;
     Rect r;
-    bool subAP = FALSE, hierS, hierD, extHierSDAttr(), swapped = FALSE;
+    bool subAP = FALSE, hierS, hierD, extHierSDAttr();
     float sdM;
     char devchar;
     bool has_model = TRUE;
@@ -507,6 +523,8 @@ spcdevHierVisit(hc, dev, scale)
 	source = drain = &dev->dev_terms[1];
     if (dev->dev_nterm >= 3)
     {
+        drain = &dev->dev_terms[2];
+
         /* If any terminal is marked with attribute "D" or "S"  */
         /* (label "D$" or "S$" at poly-diffusion interface),    */
         /* then force order of source and drain accordingly.    */
@@ -516,11 +534,8 @@ spcdevHierVisit(hc, dev, scale)
                 (dev->dev_terms[2].dterm_attrs &&
                 !strcmp(dev->dev_terms[2].dterm_attrs, "S")))
 	{
-            swapDrainSource(dev, &source, &drain);
-	    swapped = TRUE;
+            swapDrainSource(dev);
 	}
-        else
-            drain = &dev->dev_terms[2];
     }
     else if (dev->dev_nterm == 1)	// Is a device with one terminal an error?
 	source = drain = &dev->dev_terms[0];
@@ -1008,10 +1023,6 @@ spcdevHierVisit(hc, dev, scale)
 	        break;
     }
     fprintf(esSpiceF, "\n");
-
-    /* If S/D parameters were swapped, then put them back */
-    if (swapped) swapDrainSource(dev, NULL, NULL);
-
     return 0;
 }
 
@@ -1557,6 +1568,7 @@ devDistJunctHierVisit(hc, dev, scale)
  *	for each connection to it.  Note that this generates an arbitrary
  *	port order for each cell.  To have a specific port order, it is
  *	necessary to generate ports for each cell.
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -1570,6 +1582,7 @@ esMakePorts(hc, cdata)
     Use *use;
     HashEntry *he;
     EFNodeName *nn;
+
     char *name, *portname, *tptr, *aptr, *locname;
     int j;
 
@@ -1625,8 +1638,11 @@ esMakePorts(hc, cdata)
 		*tptr = '/';
 		portname = tptr + 1;
 
-		// Find the net of portname in the subcell and
-		// make it a port if it is not already.
+		/* Find the net of portname in the subcell and make it a
+		 * port if it is not already.  It is possible that the
+		 * preferred node name is in the merge list, so the merging
+		 * code may need to replace it with another name.
+		 */
 
 		if (portdef)
 		{
@@ -1880,6 +1896,8 @@ esHierVisit(hc, cdata)
 		freeMagic(p);
 	    devMergeList = NULL;
 	}
+	else if (esDistrJunct)
+	    EFHierVisitDevs(hcf, devDistJunctHierVisit, (ClientData)NULL);
 
 	/* Output devices */
 	EFHierVisitDevs(hcf, spcdevHierVisit, (ClientData)NULL);
