@@ -34,6 +34,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/utils.h"
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
+#include "tiles/tile.h"
 #include "extract/extract.h"
 
 /* Root of the tree being flattened */
@@ -66,7 +67,7 @@ bool efDevKilled();
  * 	(*subProc)(use, hierName, is_top)
  *	    Use *use;
  *	    HierName *hierName;
- *	    Boolean is_top;
+ *	    bool is_top;
  *	{
  *	}
  *
