@@ -4,16 +4,16 @@
  * Internal definitions for the procedures to flatten hierarchical
  * (.ext) circuit extraction files.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  * rcsid $Header: /usr/cvsroot/magic-8.0/extflat/extflat.h,v 1.2 2008/12/03 14:12:09 tim Exp $
@@ -40,8 +40,9 @@ typedef unsigned char U_char;
 /* Flags to control output of node names.  Stored in EFTrimFlags */
 #define	EF_TRIMGLOB		0x01	/* Delete trailing '!' from names */
 #define	EF_TRIMLOCAL		0x02	/* Delete trailing '#' from names */
-#define EF_CONVERTCOMMAS	0x04	/* Change ',' to ';' in names */
-#define EF_CONVERTEQUAL		0x08	/* Change '=' to ':' in names */
+#define EF_CONVERTCOMMA		0x04	/* Change ',' to '|' in names, else remove */
+#define EF_CONVERTEQUAL		0x08	/* Change '=' to ':' in names, else remove */
+#define EF_CONVERTBRACKETS	0x10	/* Change '[' and ']' to '_' in names */
 
 /*
  * capacitance type now set to float
@@ -118,6 +119,7 @@ typedef struct efnn
     struct efnn		*efnn_next;	/* Next name for this node */
     HierName		*efnn_hier;	/* HierName for this node */
     int			 efnn_port;	/* Port number for this node */
+    unsigned char	 efnn_refc;	/* #times referenced in hash */
 } EFNodeName;
 
 /*
@@ -141,7 +143,7 @@ typedef struct
 {
     int		 pa_area;
     int		 pa_perim;
-} PerimArea;
+} EFPerimArea;
 
 typedef struct efnhdr
 {
@@ -211,6 +213,7 @@ typedef struct efnode
 
     EFCapValue	 efnode_cap;	/* Total capacitance to ground for this node */
     int		 efnode_type;	/* Index into type table for node */
+    int		 efnode_num;	/* Number of items in efnode_hdr list */
     Rect	 efnode_loc;	/* Location of a 1x1 rect contained in this
 				 * node.  This information is provided in the
 				 * .ext file so it will be easy to map between
@@ -218,7 +221,7 @@ typedef struct efnode
 				 */
     EFAttr	*efnode_attrs;	/* Node attribute list */
     ClientData	 efnode_client;	/* For hire */
-    PerimArea	 efnode_pa[1];	/* Dummy; each node actually has
+    EFPerimArea	 efnode_pa[1];	/* Dummy; each node actually has
 				 * efNumResistClasses array elements
 				 * allocated to it.
 				 */
@@ -341,7 +344,7 @@ extern int EFGetPortMax();
 /* ------------------------- constants used by clients -------------- */
 /* This gives us a 32 or 64 dev types which should be ok */
 #define	BITSPERCHAR	8
-#define MAXDEVTYPES (sizeof(long)*BITSPERCHAR) 
+#define MAXDEVTYPES (sizeof(long)*BITSPERCHAR)
 
 /*
  * ANSI C definitions of arguments to EFvisit procedures
