@@ -259,6 +259,8 @@ extHierConnections(ha, cumFlat, oneFlat)
     /* Look for sticky labels in the child cell that are not	*/
     /* connected to any geometry.				*/
 
+    if (!(ExtOptions & EXT_DOLABELCHECK)) return;
+
     for (lab = sourceDef->cd_labels;  lab;  lab = lab->lab_next)
     {
 	CellDef *cumDef = cumFlat->et_use->cu_def;
@@ -346,6 +348,8 @@ extHierConnectFunc1(oneTile, ha)
     /* This allows the extractor to catch "sticky" labels that are not	*/
     /* attached to a physical layer in the parent cell.			*/
 
+    if (!(ExtOptions & EXT_DOLABELCHECK)) return 0;
+
     // NOTE by Tim, 9/10/2014:  This generates phantom nodes when the
     // labels are created by the "hard" node search;  I think this code
     // should be restricted to sticky labels only.  But not certain.
@@ -393,31 +397,7 @@ extHierConnectFunc1(oneTile, ha)
 		    node1->node_names = node2->node_names;
 		    freeMagic((char *) node2);
 		}
-
-#if 0
-		/* Copy this label to the parent def with a	*/
-		/* special flag, so we can output it as a node	*/
-	 	/* and then delete it.  Don't duplicate labels	*/
-		/* that are already in the parent.		*/
-
-		for (newlab = ha->ha_parentUse->cu_def->cd_labels;
-				newlab; newlab = newlab->lab_next)
-		    if (!strcmp(newlab->lab_text, lab->lab_text))
-			break;
-
-		if (newlab == NULL)
-		{
-		    n = sizeof(Label) + strlen(lab->lab_text)
-				- sizeof lab->lab_text + 1;
-		    newlab = (Label *)mallocMagic((unsigned)n);
-		    bcopy((char *)lab, (char *)newlab, (int)n);
-
-		    newlab->lab_next = ha->ha_parentUse->cu_def->cd_labels;
-		    ha->ha_parentUse->cu_def->cd_labels = newlab;
-		}
-#endif
 	    }
-
     }
     return (0);
 }
