@@ -17,6 +17,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/geofast.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "utils/stack.h"
 #include "database/database.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
@@ -29,75 +30,57 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "textio/txcommands.h"
 #include "resis/resis.h"
 
-typedef struct
-{
-    Rect		area;		/* Area to process */
-    TileTypeBitMask	*connectMask;	/* Connection mask for search */
-    TileType		dinfo;		/* Info about triangular search areas */
-} conSrArea;
-
-struct conSrArg2
-{
-    CellUse             *csa2_use;      /* Destination use */
-    TileTypeBitMask     *csa2_connect;  /* Table indicating what connects
-                                         * to what.
-                                         */
-    Rect                *csa2_bounds;   /* Area that limits the search */
-
-    conSrArea		*csa2_list;	/* List of areas to process */
-    int			csa2_top;	/* Index of next area to process */
-    int			csa2_size;	/* Max. number bins in area list */
-};
-
-#define CSA2_LIST_START_SIZE 256
-
 extern int dbcUnconnectFunc();
+extern int dbcConnectLabelFunc();
 extern int dbcConnectFuncDCS();
 #ifdef ARIEL
 extern int resSubSearchFunc();
 #endif
 
-static ResTranTile  *TransList = NULL;
+static ResDevTile  *DevList = NULL;
 static TileTypeBitMask	DiffTypeBitMask;
 TileTypeBitMask	ResSubsTypeBitMask;
 
 /* Forward declarations */
 extern void ResCalcPerimOverlap();
 
-
 /*
  * ----------------------------------------------------------------------------
  *
  * dbcConnectFuncDCS -- the same as dbcConnectFunc, except that it does
  *	some extra searching around diffusion tiles looking for
- *	transistors.
+ *	devices.
  *
  * Results:
  *	Always returns 0 to keep the search from aborting.
  *
  * Side effects:
  *	Adds a new record to the current check list. May also add new
- *	ResTranTile structures.
+ *	ResDevTile structures.
  *
  * ----------------------------------------------------------------------------
  */
 
 int
 dbcConnectFuncDCS(tile, cx)
-    Tile *tile;		
+    Tile *tile;
     TreeContext *cx;
 
 {
     struct conSrArg2	*csa2;
-    Rect 		tileArea, *srArea, tranArea, newarea;
-    ResTranTile		*thisTran;
+    Rect 		tileArea, *srArea, devArea, newarea;
+    ResDevTile		*thisDev;
     TileTypeBitMask	notConnectMask, *connectMask;
     Tile		*tp;
     TileType		t2, t1, loctype, ctype;
     TileType		dinfo = 0;
     SearchContext	*scx = cx->tc_scx;
-    int			pNum;
+    SearchContext	scx2;
+    int			i, pNum;
     CellDef		*def;
+    ExtDevice		*devptr;
+    TerminalPath	tpath;
+    char pathstring[FLATTERMSIZE];
 
     TiToRect(tile, &tileArea);
     srArea = &scx->scx_area;
@@ -107,9 +90,7 @@ dbcConnectFuncDCS(tile, cx)
 		((tileArea.r_ybot >= srArea->r_ytop-1) ||
 		(tileArea.r_ytop <= srArea->r_ybot+1)))
 	return 0;
- 
 
-    
     t1 = TiGetType(tile);
     if TTMaskHasType(&DiffTypeBitMask,t1)
     {
@@ -117,79 +98,83 @@ dbcConnectFuncDCS(tile, cx)
     for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp=RT(tp))
     {
          t2 = TiGetType(tp);
-	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
+	 devptr = ExtCurStyle->exts_device[t2];
+	 if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask),t2) &&
+	     TTMaskHasType(&(devptr->exts_deviceSDTypes[0]),t1))
 	     {
-    	          TiToRect(tp, &tranArea);
-	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
-		  ResCalcPerimOverlap(thisTran,tp);
-	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
-	          thisTran->type = TiGetType(tp);
-	          thisTran->nextTran = TransList;
-	          TransList = thisTran;
+    	          TiToRect(tp, &devArea);
+	          thisDev = (ResDevTile *) mallocMagic((unsigned)(sizeof(ResDevTile)));
+		  ResCalcPerimOverlap(thisDev,tp);
+	          GeoTransRect(&scx->scx_trans, &devArea, &thisDev->area);
+	          thisDev->type = TiGetType(tp);
+	          thisDev->nextDev = DevList;
+	          DevList = thisDev;
 	     }
     }
     /*right*/
     for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp=LB(tp))
     {
          t2 = TiGetType(tp);
-	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
+	 devptr = ExtCurStyle->exts_device[t2];
+	 if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask),t2) &&
+	     TTMaskHasType(&(devptr->exts_deviceSDTypes[0]),t1))
 	     {
-    	          TiToRect(tp, &tranArea);
-	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
-	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
-	          thisTran->type = TiGetType(tp);
-	          thisTran->nextTran = TransList;
-	          TransList = thisTran;
-		  ResCalcPerimOverlap(thisTran,tp);
+    	          TiToRect(tp, &devArea);
+	          thisDev = (ResDevTile *) mallocMagic((unsigned)(sizeof(ResDevTile)));
+	          GeoTransRect(&scx->scx_trans, &devArea, &thisDev->area);
+	          thisDev->type = TiGetType(tp);
+	          thisDev->nextDev = DevList;
+	          DevList = thisDev;
+		  ResCalcPerimOverlap(thisDev,tp);
 	     }
     }
     /*top*/
     for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp=BL(tp))
     {
          t2 = TiGetType(tp);
-	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
+	 devptr = ExtCurStyle->exts_device[t2];
+	 if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask),t2) &&
+	     TTMaskHasType(&(devptr->exts_deviceSDTypes[0]),t1))
 	     {
-    	          TiToRect(tp, &tranArea);
-	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
-	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
-	          thisTran->type = TiGetType(tp);
-	          thisTran->nextTran = TransList;
-	          TransList = thisTran;
-		  ResCalcPerimOverlap(thisTran,tp);
+    	          TiToRect(tp, &devArea);
+	          thisDev = (ResDevTile *) mallocMagic((unsigned)(sizeof(ResDevTile)));
+	          GeoTransRect(&scx->scx_trans, &devArea, &thisDev->area);
+	          thisDev->type = TiGetType(tp);
+	          thisDev->nextDev = DevList;
+	          DevList = thisDev;
+		  ResCalcPerimOverlap(thisDev,tp);
 	     }
     }
     /*bottom */
     for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp=TR(tp))
     {
          t2 = TiGetType(tp);
-	 if (TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	     TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
+	 devptr = ExtCurStyle->exts_device[t2];
+	 if (TTMaskHasType(&(ExtCurStyle->exts_deviceMask),t2) &&
+	     TTMaskHasType(&(devptr->exts_deviceSDTypes[0]),t1))
 	     {
-    	          TiToRect(tp, &tranArea);
-	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
-	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
-	          thisTran->type = TiGetType(tp);
-	          thisTran->nextTran = TransList;
-	          TransList = thisTran;
-		  ResCalcPerimOverlap(thisTran,tp);
+    	          TiToRect(tp, &devArea);
+	          thisDev = (ResDevTile *) mallocMagic((unsigned)(sizeof(ResDevTile)));
+	          GeoTransRect(&scx->scx_trans, &devArea, &thisDev->area);
+	          thisDev->type = TiGetType(tp);
+	          thisDev->nextDev = DevList;
+	          DevList = thisDev;
+		  ResCalcPerimOverlap(thisDev,tp);
 	     }
     }
     }
-    else if TTMaskHasType(&(ExtCurStyle->exts_transMask),t1)
+    else if TTMaskHasType(&(ExtCurStyle->exts_deviceMask),t1)
     {
-    	          TiToRect(tile, &tranArea);
-	          thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
-		  ResCalcPerimOverlap(thisTran,tile);
-	          GeoTransRect(&scx->scx_trans, &tranArea, &thisTran->area);
-	          thisTran->type = TiGetType(tile);
-	          thisTran->nextTran = TransList;
-	          TransList = thisTran;
+    	          TiToRect(tile, &devArea);
+	          thisDev = (ResDevTile *) mallocMagic((unsigned)(sizeof(ResDevTile)));
+		  ResCalcPerimOverlap(thisDev,tile);
+	          GeoTransRect(&scx->scx_trans, &devArea, &thisDev->area);
+	          thisDev->type = TiGetType(tile);
+	          thisDev->nextDev = DevList;
+	          DevList = thisDev;
     }
     /* in some cases (primarily bipolar technology), we'll want to extract
-       transistors whose substrate terminals are part of the given region.
+       devices whose substrate terminals are part of the given region.
        The following does that check.  (10-11-88)
     */
 #ifdef ARIEL
@@ -201,8 +186,8 @@ dbcConnectFuncDCS(tile, cx)
          {
 	     if (TTMaskIntersect(&DBPlaneTypes[pNum], mask))
 	     {
-	          (void)DBSrPaintArea((Tile *) NULL, 
-		  	scx->scx_use->cu_def->cd_planes[pNum], 
+	          (void)DBSrPaintArea((Tile *) NULL,
+		  	scx->scx_use->cu_def->cd_planes[pNum],
 		        &tileArea,mask,resSubSearchFunc, (ClientData) cx);
 	     }
          }
@@ -230,7 +215,7 @@ dbcConnectFuncDCS(tile, cx)
     if (DBIsContact(loctype))
     {
 	/* The mask of contact types must include all stacked contacts */
-	
+
 	TTMaskZero(&notConnectMask);
 	TTMaskSetMask(&notConnectMask, &DBNotConnectTbl[loctype]);
     }
@@ -250,6 +235,21 @@ dbcConnectFuncDCS(tile, cx)
 		&newarea, DBStdPaintTbl(loctype, pNum),
 		(PaintUndoInfo *) NULL);
 
+    /* Check the source def for any labels belonging to this	*/
+    /* tile area and plane, and add them to the destination	*/
+
+    scx2 = *csa2->csa2_topscx;
+    scx2.scx_area = newarea;
+
+    pathstring[0] = '\0';
+    tpath.tp_first = tpath.tp_next = pathstring;
+    tpath.tp_last = pathstring + FLATTERMSIZE;
+
+    DBTreeSrLabels(&scx2, connectMask, csa2->csa2_xMask, &tpath,
+    		TF_LABEL_ATTACH, dbcConnectLabelFunc,
+    		(ClientData)csa2);
+    // DBCellCopyLabels(&scx2, connectMask, csa2->csa2_xMask, csa2->csa2_use, NULL);
+
     /* Only extend those sides bordering the diagonal tile */
 
     if (dinfo & TT_DIAGONAL)
@@ -272,29 +272,29 @@ dbcConnectFuncDCS(tile, cx)
 	newarea.r_ytop += 1;
     }
 
+    /* Check if any of the last 5 entries has the same type and */
+    /* area.  If so, don't duplicate the existing entry.        */
+    /* (NOTE:  Connect masks are all from the same table, so    */
+    /* they can be compared by address, no need for TTMaskEqual)*/
+
+    for (i = csa2->csa2_lasttop; (i >= 0) && (i > csa2->csa2_lasttop - 5); i--)
+        if (connectMask == csa2->csa2_list[i].connectMask)
+            if (GEO_SURROUND(&csa2->csa2_list[i].area, &newarea))
+                return 0;
+
     /* Register the area and connection mask as needing to be processed */
 
-    if (++csa2->csa2_top == csa2->csa2_size)
+    if (++csa2->csa2_top == CSA2_LIST_SIZE)
     {
 	/* Reached list size limit---need to enlarge the list	   */
 	/* Double the size of the list every time we hit the limit */
 
 	conSrArea *newlist;
-	int i, lastsize = csa2->csa2_size;
 
-	csa2->csa2_size *= 2;
-
-	newlist = (conSrArea *)mallocMagic(csa2->csa2_size * sizeof(conSrArea));
-	memcpy((void *)newlist, (void *)csa2->csa2_list,
-			(size_t)lastsize * sizeof(conSrArea));
-	// for (i = 0; i < lastsize; i++)
-	// {
-	//     newlist[i].area = csa2->csa2_list[i].area;
-	//     newlist[i].connectMask = csa2->csa2_list[i].connectMask;
-	//     newlist[i].dinfo = csa2->csa2_list[i].dinfo;
-	// }
-	freeMagic((char *)csa2->csa2_list);
+	newlist = (conSrArea *)mallocMagic(CSA2_LIST_SIZE * sizeof(conSrArea));
+	StackPush((ClientData)csa2->csa2_list, csa2->csa2_stack);
 	csa2->csa2_list = newlist;
+	csa2->csa2_top = 0;
     }
 
     csa2->csa2_list[csa2->csa2_top].area = newarea;
@@ -304,7 +304,7 @@ dbcConnectFuncDCS(tile, cx)
     return 0;
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -319,18 +319,18 @@ dbcConnectFuncDCS(tile, cx)
  */
 
 void
-ResCalcPerimOverlap(trans, tile)
-    ResTranTile	*trans;
+ResCalcPerimOverlap(dev, tile)
+    ResDevTile	*dev;
     Tile	*tile;
 
 {
     Tile	*tp;
     int		t1;
     int		overlap;
-    
-    trans->perim = (TOP(tile)-BOTTOM(tile)-LEFT(tile)+RIGHT(tile))<<1;
+
+    dev->perim = (TOP(tile)-BOTTOM(tile)-LEFT(tile)+RIGHT(tile))<<1;
     overlap =0;
-    
+
     t1 = TiGetType(tile);
     /* left */
     for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp=RT(tp))
@@ -340,7 +340,7 @@ ResCalcPerimOverlap(trans, tile)
 	      overlap += MIN(TOP(tile),TOP(tp))-
 	   		  MAX(BOTTOM(tile),BOTTOM(tp));
 	}
-    	 
+
     }
     /*right*/
     for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp=LB(tp))
@@ -350,7 +350,7 @@ ResCalcPerimOverlap(trans, tile)
 	      overlap += MIN(TOP(tile),TOP(tp))-
 	   		  MAX(BOTTOM(tile),BOTTOM(tp));
 	}
-    	 
+
     }
     /*top*/
     for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp=BL(tp))
@@ -360,7 +360,7 @@ ResCalcPerimOverlap(trans, tile)
 	      overlap += MIN(RIGHT(tile),RIGHT(tp))-
 	   		  MAX(LEFT(tile),LEFT(tp));
 	}
-    	 
+
     }
     /*bottom */
     for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp=TR(tp))
@@ -370,22 +370,22 @@ ResCalcPerimOverlap(trans, tile)
 	      overlap += MIN(RIGHT(tile),RIGHT(tp))-
 	   		  MAX(LEFT(tile),LEFT(tp));
 	}
-    	 
+
     }
-    trans->overlap = overlap;
+    dev->overlap = overlap;
 }
-	
-
+
+
 /*
  * ----------------------------------------------------------------------------
  *
  * DBTreeCopyConnectDCS --
  *
- * 	Basically the same as DBTreeCopyConnect, except it calls 
+ * 	Basically the same as DBTreeCopyConnect, except it calls
  *	dbcConnectFuncDCS.
  *
  * Results:
- *	Linked list of transistors.
+ *	Linked list of devices.
  *
  * Side effects:
  *	The contents of the result cell are modified.
@@ -393,60 +393,80 @@ ResCalcPerimOverlap(trans, tile)
  * ----------------------------------------------------------------------------
  */
 
-ResTranTile *
+ResDevTile *
 DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
     SearchContext *scx;
     TileTypeBitMask *mask;
-    int xMask;	
+    int xMask;
     TileTypeBitMask *connect;
-    Rect *area;	
+    Rect *area;
     CellUse *destUse;
 
 {
     static int 		first = 1;
-    struct conSrArg2	csa2; 
-    int			tran, pNum;
-    char		*tran_name;
+    struct conSrArg2	csa2;
+    int			dev, pNum;
+    char		*dev_name;
     TileTypeBitMask	*newmask;
-    ResTranTile		*CurrentT;
+    ResDevTile		*CurrentT;
     CellDef		*def = destUse->cu_def;
     TileType		newtype;
+    ExtDevice		*devptr;
 
     csa2.csa2_use = destUse;
+    csa2.csa2_xMask = xMask;
     csa2.csa2_bounds = area;
     csa2.csa2_connect = connect;
+    csa2.csa2_topscx = scx;
 
-    csa2.csa2_size = CSA2_LIST_START_SIZE;
-    csa2.csa2_list = (conSrArea *)mallocMagic(CSA2_LIST_START_SIZE
-		* sizeof(conSrArea));
+    csa2.csa2_list = (conSrArea *)mallocMagic(CSA2_LIST_SIZE * sizeof(conSrArea));
     csa2.csa2_top = -1;
+    csa2.csa2_lasttop = -1;
+
+    csa2.csa2_stack = StackNew(100);
 
     if (first)
     {
 	TTMaskZero(&DiffTypeBitMask);
 	TTMaskZero(&ResSubsTypeBitMask);
-	for (tran = TT_TECHDEPBASE; tran < TT_MAXTYPES; tran++)
+	for (dev = TT_TECHDEPBASE; dev < TT_MAXTYPES; dev++)
 	{
-	    tran_name = (ExtCurStyle->exts_transName)[tran];
-	    if ((tran_name != NULL) && (strcmp(tran_name, "None")))
+	    devptr = ExtCurStyle->exts_device[dev];
+	    if ((devptr != NULL) && ((dev_name = devptr->exts_deviceName) != NULL)
+		    && (strcmp(dev_name, "None")))
 	    {
 		TTMaskSetMask(&DiffTypeBitMask,
-	      		&(ExtCurStyle->exts_transSDTypes[tran][0]));
+	      		&(devptr->exts_deviceSDTypes[0]));
 		TTMaskSetMask(&ResSubsTypeBitMask,
-	      		&(ExtCurStyle->exts_transSubstrateTypes[tran]));
+	      		&(devptr->exts_deviceSubstrateTypes));
 	    }
 	}
 	first = 0;
     }
 
-    TransList = NULL;
+    DevList = NULL;
     DBTreeSrTiles(scx, mask, xMask, dbcConnectFuncDCS, (ClientData) &csa2);
     while (csa2.csa2_top >= 0)
     {
 	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
 	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
 	newtype = csa2.csa2_list[csa2.csa2_top].dinfo;
-	csa2.csa2_top--;
+	if (csa2.csa2_top == 0)
+	{
+	    if (StackLook(csa2.csa2_stack) != (ClientData)NULL)
+	    {
+		freeMagic(csa2.csa2_list);
+		csa2.csa2_list = (conSrArea *)StackPop(csa2.csa2_stack);
+		csa2.csa2_top = CSA2_LIST_SIZE - 1;
+	    }
+	    else
+		csa2.csa2_top--;
+	}
+	else
+	    csa2.csa2_top--;
+
+	csa2.csa2_lasttop = csa2.csa2_top;
+
 	if (newtype & TT_DIAGONAL)
 	    DBTreeSrNMTiles(scx, newtype, newmask, xMask, dbcConnectFuncDCS,
 			(ClientData) &csa2);
@@ -454,8 +474,9 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
 	    DBTreeSrTiles(scx, newmask, xMask, dbcConnectFuncDCS, (ClientData) &csa2);
     }
     freeMagic((char *)csa2.csa2_list);
+    StackFree(csa2.csa2_stack);
 
-    for (CurrentT = TransList; CurrentT != NULL; CurrentT=CurrentT->nextTran)
+    for (CurrentT = DevList; CurrentT != NULL; CurrentT=CurrentT->nextDev)
     {
 	TileType  t = CurrentT->type;
 	TileType  nt;
@@ -473,9 +494,9 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
     }
 
     DBReComputeBbox(def);
-    return(TransList);
+    return(DevList);
 }
-
+
 
 #ifdef ARIEL
 /*
@@ -483,7 +504,7 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
  *
  * resSubSearchFunc --
  *
- *	called when DBSrPaintArea finds a transistor within
+ *	called when DBSrPaintArea finds a device within
  *	a substrate area.
  *
  * Results:
@@ -498,26 +519,28 @@ int
 resSubSearchFunc(tile,cx)
 	Tile	*tile;
 	TreeContext	*cx;
-	
+
 
 {
-     ResTranTile	*thisTran;
-     Rect		tranArea;
+     ResDevTile		*thisDev;
+     Rect		devArea;
      TileType		t = TiGetType(tile);
+     ExtDevice		*devptr;
 
-     /* Right now, we're only going to extract substrate terminals for 
+     /* Right now, we're only going to extract substrate terminals for
      	devices with only one diffusion terminal, principally bipolar
 	devices.
      */
-     if (ExtCurStyle->exts_transSDCount[t] >1) return 0;
-     TiToRect(tile, &tranArea);
-     thisTran = (ResTranTile *) mallocMagic((unsigned)(sizeof(ResTranTile)));
-     GeoTransRect(&cx->tc_scx->scx_trans, &tranArea, &thisTran->area);
-     thisTran->type = t;
-     thisTran->nextTran = TransList;
-     TransList = thisTran;
-     ResCalcPerimOverlap(thisTran,tile);
-     
+     devptr = ExtCurStyle->exts_device[t]
+     if (devptr->exts_deviceSDCount >1) return 0;
+     TiToRect(tile, &devArea);
+     thisDev = (ResDevTile *) mallocMagic((unsigned)(sizeof(ResDevTile)));
+     GeoTransRect(&cx->tc_scx->scx_trans, &devArea, &thisDev->area);
+     thisDev->type = t;
+     thisDev->nextDev = DevList;
+     DevList = thisDev;
+     ResCalcPerimOverlap(thisDev,tile);
+
      return 0;
 }
 
